@@ -85,17 +85,10 @@ ggv_control_destroy (BonoboObject *object)
 									control->priv->gconf_notify_id);
 		control->priv->gconf_notify_id = 0;
 	}
-
-#if 0
 	if (control->priv->zoom_control) {
 		bonobo_object_unref(BONOBO_OBJECT(control->priv->zoom_control));
 		control->priv->zoom_control = NULL;
 	}
-	if(control->priv->uic) {
-		bonobo_object_unref(BONOBO_OBJECT(control->priv->uic));
-		control->priv->uic = NULL;
-	}
-#endif
 
 	if(BONOBO_OBJECT_CLASS (ggv_control_parent_class)->destroy)
 		BONOBO_OBJECT_CLASS (ggv_control_parent_class)->destroy (object);
@@ -510,6 +503,7 @@ _set_zoom_items_sensitivity(GgvControl *control, BonoboUIComponent *uic, gboolea
 	if (container == CORBA_OBJECT_NIL)
 		return;
 
+	bonobo_ui_component_freeze(uic, NULL);
 	bonobo_ui_component_set_prop(uic, "/commands/ZoomIn",
 								 "sensitive", prop_val, NULL);
 	bonobo_ui_component_set_prop(uic, "/commands/ZoomOut",
@@ -530,6 +524,7 @@ _set_zoom_items_sensitivity(GgvControl *control, BonoboUIComponent *uic, gboolea
 								 "sensitive", prop_val, NULL);
 	bonobo_ui_component_set_prop(uic, "/commands/Zoom0401",
 								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_thaw(uic, NULL);
 }
 
 static void
@@ -626,9 +621,6 @@ ggv_control_create_ui (GgvControl *control)
 		bonobo_ui_component_object_set(control->priv->uic, "/Sidebar/Zoom Control Placeholder/ZoomControl",
 									   BONOBO_OBJREF(zoom_control), NULL);
 		control->priv->zoom_control = zoom_control;
-#if 1
-		bonobo_object_unref(BONOBO_OBJECT(zoom_control));
-#endif
 		control->priv->zoom_adj = zoom_adj;
 	}
 }
