@@ -50,11 +50,6 @@ ggv_sidebar_destroy (BonoboObject *object)
 
 	sidebar = GGV_SIDEBAR (object);
 
-	if (sidebar->priv->root) {
-		gtk_widget_unref (sidebar->priv->root);
-		sidebar->priv->root = NULL;
-	}
-
 	if(BONOBO_OBJECT_CLASS (ggv_sidebar_parent_class)->destroy)
 		BONOBO_OBJECT_CLASS (ggv_sidebar_parent_class)->destroy (object);
 }
@@ -122,99 +117,120 @@ cl_toggle_even(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpoint
 static gboolean
 cl_toggle_odd(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
 {
-        gint *n = (gint *)data;
-        if(*n%2 != 0) {
-                gboolean val;
-                gtk_tree_model_get(model, iter, 0, &val, -1);
-                gtk_list_store_set(GTK_LIST_STORE(model), iter, 0, !val, -1);
-        }
-        (*n)++;
-        return FALSE;
+	gint *n = (gint *)data;
+	if(*n%2 != 0) {
+		gboolean val;
+		gtk_tree_model_get(model, iter, 0, &val, -1);
+		gtk_list_store_set(GTK_LIST_STORE(model), iter, 0, !val, -1);
+	}
+	(*n)++;
+	return FALSE;
 }
 
 static gboolean
 cl_clear_all(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
 {
-        gtk_list_store_set(GTK_LIST_STORE(model), iter, 0, FALSE, -1);
-        return FALSE;
+	gtk_list_store_set(GTK_LIST_STORE(model), iter, 0, FALSE, -1);
+	return FALSE;
 }
 
 static void
 toggle_all_clicked(GtkWidget *widget, gpointer data)
 {
-        GgvSidebar *sidebar = GGV_SIDEBAR(data);
-        gtk_tree_model_foreach(GTK_TREE_MODEL(GTK_CHECK_LIST(sidebar->priv->checklist)->list_model),
-                               cl_toggle_all, NULL);
+	GgvSidebar *sidebar = GGV_SIDEBAR(data);
+	gtk_tree_model_foreach(GTK_TREE_MODEL(GTK_CHECK_LIST(sidebar->priv->checklist)->list_model),
+						   cl_toggle_all, NULL);
 }
 
 static void
 toggle_odd_clicked(GtkWidget *widget, gpointer data)
 {
-        gint n = 1;
-        GgvSidebar *sidebar = GGV_SIDEBAR(data);
-        gtk_tree_model_foreach(GTK_TREE_MODEL(GTK_CHECK_LIST(sidebar->priv->checklist)->list_model),
-                               cl_toggle_odd, &n);
+	gint n = 1;
+	GgvSidebar *sidebar = GGV_SIDEBAR(data);
+	gtk_tree_model_foreach(GTK_TREE_MODEL(GTK_CHECK_LIST(sidebar->priv->checklist)->list_model),
+						   cl_toggle_odd, &n);
 }
 
 static void
 toggle_even_clicked(GtkWidget *widget, gpointer data)
 {
-        GgvSidebar *sidebar = GGV_SIDEBAR(data);
-        gint n = 1;
-        gtk_tree_model_foreach(GTK_TREE_MODEL(GTK_CHECK_LIST(sidebar->priv->checklist)->list_model),
-                               cl_toggle_even, &n);
+	GgvSidebar *sidebar = GGV_SIDEBAR(data);
+	gint n = 1;
+	gtk_tree_model_foreach(GTK_TREE_MODEL(GTK_CHECK_LIST(sidebar->priv->checklist)->list_model),
+						   cl_toggle_even, &n);
 }
 
 static void
 clear_all_clicked(GtkWidget *widget, gpointer data)
 {
-        GgvSidebar *sidebar = GGV_SIDEBAR(data);
-        gtk_tree_model_foreach(GTK_TREE_MODEL(GTK_CHECK_LIST(sidebar->priv->checklist)->list_model),
-                               cl_clear_all, NULL);
+	GgvSidebar *sidebar = GGV_SIDEBAR(data);
+	gtk_tree_model_foreach(GTK_TREE_MODEL(GTK_CHECK_LIST(sidebar->priv->checklist)->list_model),
+						   cl_clear_all, NULL);
 }
 
 static void
 page_list_selection_changed(GtkTreeSelection *sel,
                             gpointer user_data)
 {
-        GgvSidebar *sidebar = GGV_SIDEBAR(user_data);
-        GtkTreeIter iter;
-        GtkTreeModel *model;
-
-        if(gtk_tree_selection_get_selected(sel, &model, &iter)) {
-                GtkTreePath *path;
-                gint page;
-                gchar *path_str;
-
-                path = gtk_tree_model_get_path(model,
-                                               &iter);
-                path_str = gtk_tree_path_to_string(path);
-                page = atoi(path_str);
-                g_free(path_str);
-                if(page >= 0)
-					ggv_postscript_view_goto_page(sidebar->priv->ps_view, page);
-        }
+	GgvSidebar *sidebar = GGV_SIDEBAR(user_data);
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+	
+	if(gtk_tree_selection_get_selected(sel, &model, &iter)) {
+		GtkTreePath *path;
+		gint page;
+		gchar *path_str;
+		
+		path = gtk_tree_model_get_path(model,
+									   &iter);
+		path_str = gtk_tree_path_to_string(path);
+		page = atoi(path_str);
+		g_free(path_str);
+		if(page >= 0)
+			ggv_postscript_view_goto_page(sidebar->priv->ps_view, page);
+	}
 }
 
 void
 ggv_sidebar_create_page_list(GgvSidebar *sidebar)
 {
-        gint page_count, i;
-        gchar **page_names;
+	gint page_count, i;
+	gchar **page_names;
+#if 0
+	gboolean sel_path;
+	GtkTreePath *path;
+	GtkTreeSelection *sel;
+	GtkTreeIter iter;
 
-        gtk_check_list_clear(GTK_CHECK_LIST(sidebar->priv->checklist));
+	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(sidebar->priv->checklist));
+	sel_path = gtk_tree_selection_get_selected(sel, NULL, &iter);
+	if(sel_path)
+		path = gtk_tree_model_get_path(GTK_TREE_MODEL(GTK_CHECK_LIST(sidebar->priv->checklist)->list_model),
+									   &iter);
+	else
+		path = NULL;
+#endif
 
-        page_count = ggv_postscript_view_get_page_count(sidebar->priv->ps_view);
-		page_names = ggv_postscript_view_get_page_names(sidebar->priv->ps_view);
-        if(page_names == NULL) {
-                g_warning("Unable to fetch page names from GgvPostScriptView.");
-                return;
-        }
-        for(i = 0; page_names[i] != NULL; i++) {
-                gtk_check_list_append_row(GTK_CHECK_LIST(sidebar->priv->checklist),
-                                          page_names[i]);
-        }
-		g_strfreev(page_names);
+	gtk_check_list_clear(GTK_CHECK_LIST(sidebar->priv->checklist));
+
+	page_count = ggv_postscript_view_get_page_count(sidebar->priv->ps_view);
+	page_names = ggv_postscript_view_get_page_names(sidebar->priv->ps_view);
+	if(page_count <= 0)
+		return;
+	if(page_names == NULL)
+		return;
+	for(i = 0; page_names[i] != NULL; i++) {
+		gtk_check_list_append_row(GTK_CHECK_LIST(sidebar->priv->checklist),
+								  page_names[i]);
+	}
+	g_strfreev(page_names);
+#if 0
+	if(path) {
+		sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(sidebar->priv->checklist));
+		gtk_tree_selection_select_path(sel, path);
+		gtk_tree_path_free(path);
+	}
+#endif
 }
 
 void
@@ -223,9 +239,9 @@ ggv_sidebar_page_changed(GgvSidebar *sidebar, gint page)
 	GtkTreeSelection *sel;
 	gchar path_str[16];
 	GtkTreePath *path;
-	gint current_page;
 
-	current_page = page;
+	if(page < 0)
+		page = 0;
 	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(sidebar->priv->checklist));
 	g_snprintf(path_str, 15, "%d", page);
 	path = gtk_tree_path_new_from_string(path_str);
@@ -245,7 +261,7 @@ BONOBO_TYPE_FUNC (GgvSidebar, BONOBO_TYPE_CONTROL, ggv_sidebar);
 GgvSidebar *
 ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 {
-	GtkWidget *view, *hbox, *image, *sw;
+	GtkWidget *hbox, *image, *sw;
 	GtkTreeSelection *sel;
 
 	g_return_val_if_fail (ps_view != NULL, NULL);
