@@ -23,8 +23,6 @@ G_BEGIN_DECLS
 #define DEFAULT_FILE_SEL_WIDTH   320
 #define DEFAULT_FILE_SEL_HEIGHT  256
 
-typedef void (*GgvPrefsCb)(gpointer data);
-
 typedef enum {
         GGV_TOOLBAR_STYLE_DEFAULT = 0,
         GGV_TOOLBAR_STYLE_BOTH = 1,
@@ -51,8 +49,6 @@ extern gint ggv_default_height, file_sel_height;
 void         ggv_prefs_load (void);
 void         ggv_prefs_save (void);
 GConfClient *ggv_prefs_gconf_client (void);
-void         ggv_prefs_add_callback (GgvPrefsCb cb, gpointer data);
-void         ggv_prefs_remove_callback (gpointer data);
 
 G_END_DECLS
 
