@@ -8,6 +8,7 @@
  */
 
 #ifndef __GGV_WINDOW_H__
+#define __GGV_WINDOW_H__
 
 #include <config.h>
 
@@ -84,6 +85,7 @@ GtkWidget *ggv_window_new               (void);
 void      ggv_window_close              (GgvWindow *win);
 gboolean  ggv_window_load               (GgvWindow *win, const gchar *filename);
 void      ggv_window_apply_prefs        (GgvWindow *win);
+void      ggv_window_sync_toolbar_style (GgvWindow *win);
 const GList *ggv_get_window_list(void);
 
 G_END_DECLS
