@@ -16,6 +16,7 @@
 #include <bonobo.h>
 #include <bonobo/bonobo-ui-main.h>
 
+#include <signal.h>
 #include <math.h>
 #include <ctype.h>
 
@@ -26,6 +27,16 @@
 
 static gint win_count = 0;
 
+static void
+sig_handler(int signum)
+{
+        const GList *l;
+
+        for(l = ggv_get_window_list(); l != NULL; l = l->next)
+                gtk_widget_destroy(GTK_WIDGET(l->data));
+        bonobo_main_quit();
+}
+
 static GgvWindow *
 create_window(const gchar *filename)
 {
@@ -66,7 +77,7 @@ create_windows_from_cmdline(gpointer data)
 static void
 client_die(GnomeClient *client, gpointer data)
 {
-        gtk_main_quit ();
+        bonobo_main_quit ();
 }
 
 static gint
@@ -111,7 +122,6 @@ int
 main(int argc, char **argv)
 {
 	CORBA_Environment ev;
-	CORBA_ORB orb;
 	GError *error;
 	poptContext ctx;
 	GValue value = { 0, };
@@ -129,6 +139,9 @@ main(int argc, char **argv)
 	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
 	textdomain(GETTEXT_PACKAGE);
 
+        signal(SIGTERM, sig_handler);
+        signal(SIGINT, sig_handler);
+
 	program = gnome_program_init("ggv", VERSION,
                                      LIBGNOMEUI_MODULE,
                                      argc, argv,
@@ -161,8 +174,6 @@ main(int argc, char **argv)
 
         gtk_idle_add (create_windows_from_cmdline, &ctx);
 
-        ggv_prefs_add_callback((GgvPrefsCb)ggv_window_apply_prefs, NULL);
-
 	gnome_window_icon_set_default_from_file (GNOMEICONDIR "/gnome-ghostview.png");
 
         ggv_prefs_load();
