@@ -42,7 +42,7 @@
  *		
  *			<plass@thep.physik.uni-mainz.de>
  */
- 
+
 /* end of patch */
 
 #include <stdlib.h>
@@ -58,6 +58,7 @@
 #include "gtkgs.h"
 #include "gsdefaults.h"
 #include "ps.h"
+#include "gsio.h"
 
 #ifdef BSD4_2
 #define memset(a,b,c) bzero(a,c)
@@ -100,7 +101,7 @@ static int  blank();
  *	This routine returns a pointer to the document structure.
  *	The structure contains the information relevant to previewing.
  *      These include EPSF flag (to tell if the file is a encapsulated figure),
- *      Page Media (for the Page Size), Bounding Box (to minimize backing
+ *      Page Size (for the Page Size), Bounding Box (to minimize backing
  *      pixmap size or determine window size for encapsulated PostScript), 
  *      Orientation of Paper (for default transformation matrix), and
  *      Page Order.  The title and CreationDate are also retrieved to
@@ -209,7 +210,7 @@ struct document *psscan (FILE *file, int respect_eof)
     int page_order_set = NONE;
     int orientation_set = NONE;
     int page_bb_set = NONE;
-    int page_media_set = NONE;
+    int page_size_set = NONE;
     int preread;		/* flag which tells the readline isn't needed */
     int i;
     unsigned int maxpages = 0;
@@ -343,12 +344,16 @@ struct document *psscan (FILE *file, int respect_eof)
 	    if (strcmp(text, "(atend)") == 0) {
 		orientation_set = ATEND;
 	    } else if (strcmp(text, "Portrait") == 0) {
-		doc->orientation = PORTRAIT;
+		doc->orientation = GTK_GS_ORIENTATION_PORTRAIT;
 		orientation_set = 1;
 	    } else if (strcmp(text, "Landscape") == 0) {
-		doc->orientation = LANDSCAPE;
+		doc->orientation = GTK_GS_ORIENTATION_LANDSCAPE;
 		orientation_set = 1;
 	    }
+            else if(strcmp(text, "Seascape") == 0) {
+                    doc->orientation = GTK_GS_ORIENTATION_SEASCAPE;
+                    orientation_set = 1;
+            }
 	} else if (page_order_set == NONE && iscomment(line+2, "PageOrder:")) {
 	    sscanf(line+length("%%PageOrder:"), "%s", text);
 	    if (strcmp(text, "(atend)") == 0) {
@@ -388,95 +393,95 @@ struct document *psscan (FILE *file, int respect_eof)
 			    doc->pages = g_new0 (struct page, maxpages);
 		}
 	    }
-	} else if (doc->nummedia == NONE &&
+	} else if (doc->numsizes == NONE &&
 		   iscomment(line+2, "DocumentMedia:")) {
 	    float w, h;
-	    doc->media = g_new0 (GtkGSPaperSize, 1);
-	    doc->media[0].name = get_next_text(line+length("%%DocumentMedia:"),
+	    doc->size = g_new0 (GtkGSPaperSize, 1);
+	    doc->size[0].name = get_next_text(line+length("%%DocumentMedia:"),
 					 &next_char);
-	    if (doc->media[0].name != NULL) {
+	    if (doc->size[0].name != NULL) {
 		if (sscanf(next_char, "%f %f", &w, &h) == 2) {
-		    doc->media[0].width = w + 0.5;
-		    doc->media[0].height = h + 0.5;
+		    doc->size[0].width = w + 0.5;
+		    doc->size[0].height = h + 0.5;
 		}
-		if (doc->media[0].width != 0 && doc->media[0].height != 0)
-		    doc->nummedia = 1;
+		if (doc->size[0].width != 0 && doc->size[0].height != 0)
+		    doc->numsizes = 1;
 		else
-		    g_free(doc->media[0].name);
+		    g_free(doc->size[0].name);
 	    }
 	    preread=1;
 	    while (readline(line, sizeof line, file, &position, &line_len) &&
 		   DSCcomment(line) && iscomment(line+2, "+")) {
 		section_len += line_len;
-		doc->media = g_renew (GtkGSPaperSize,
-				      doc->media, doc->nummedia+1);
-		doc->media[doc->nummedia].name = get_next_text(line+length("%%+"),
-							 &next_char);
-		if (doc->media[doc->nummedia].name != NULL) {
+		doc->size = g_renew (GtkGSPaperSize,
+				     doc->size, doc->numsizes+1);
+		doc->size[doc->numsizes].name = get_next_text(line+length("%%+"),
+							      &next_char);
+		if (doc->size[doc->numsizes].name != NULL) {
 		    if (sscanf(next_char, "%f %f", &w, &h) == 2) {
-			doc->media[doc->nummedia].width = w + 0.5;
-			doc->media[doc->nummedia].height = h + 0.5;
+			doc->size[doc->numsizes].width = w + 0.5;
+			doc->size[doc->numsizes].height = h + 0.5;
 		    }
-		    if (doc->media[doc->nummedia].width != 0 &&
-			doc->media[doc->nummedia].height != 0) doc->nummedia++;
+		    if (doc->size[doc->numsizes].width != 0 &&
+			doc->size[doc->numsizes].height != 0) doc->numsizes++;
 		    else
-			g_free(doc->media[doc->nummedia].name);
+			g_free(doc->size[doc->numsizes].name);
 		}
 	    }
 	    section_len += line_len;
-	    if (doc->nummedia != 0) doc->default_page_media = doc->media;
-	} else if (doc->nummedia == NONE &&
+	    if (doc->numsizes != 0) doc->default_page_size = doc->size;
+	} else if (doc->numsizes == NONE &&
 		   iscomment(line+2, "DocumentPaperSizes:")) {
 
-	    doc->media = g_new0 (GtkGSPaperSize, 1);
-	    doc->media[0].name = get_next_text(line+length("%%DocumentPaperSizes:"),
-					 &next_char);
-	    if (doc->media[0].name != NULL) {
-		doc->media[0].width = 0;
-		doc->media[0].height = 0;
+	    doc->size = g_new0 (GtkGSPaperSize, 1);
+	    doc->size[0].name = get_next_text(line+length("%%DocumentPaperSizes:"),
+					      &next_char);
+	    if (doc->size[0].name != NULL) {
+		doc->size[0].width = 0;
+		doc->size[0].height = 0;
 		for (dmp=papersizes; dmp->name != NULL; dmp++) {
 		    /* Note: Paper size comment uses down cased paper size
 		     * name.  Case insensitive compares are only used for
 		     * PaperSize comments.
 		     */
-		    if (strcasecmp(doc->media[0].name, dmp->name) == 0) {
-			g_free(doc->media[0].name);
-			doc->media[0].name = g_strdup (dmp->name);
-			doc->media[0].width = dmp->width;
-			doc->media[0].height = dmp->height;
+		    if (strcasecmp(doc->size[0].name, dmp->name) == 0) {
+			g_free(doc->size[0].name);
+			doc->size[0].name = g_strdup (dmp->name);
+			doc->size[0].width = dmp->width;
+			doc->size[0].height = dmp->height;
 			break;
 		    }
 		}
-		if (doc->media[0].width != 0 && doc->media[0].height != 0)
-		    doc->nummedia = 1;
+		if (doc->size[0].width != 0 && doc->size[0].height != 0)
+		    doc->numsizes = 1;
 		else
-		    g_free(doc->media[0].name);
+		    g_free(doc->size[0].name);
 	    }
 	    while ( (cp = get_next_text(next_char, &next_char)) ) {
-		doc->media = g_renew (GtkGSPaperSize,
-				      doc->media, doc->nummedia+1);
-		doc->media[doc->nummedia].name = cp;
-		doc->media[doc->nummedia].width = 0;
-		doc->media[doc->nummedia].height = 0;
+		doc->size = g_renew (GtkGSPaperSize,
+				     doc->size, doc->numsizes+1);
+		doc->size[doc->numsizes].name = cp;
+		doc->size[doc->numsizes].width = 0;
+		doc->size[doc->numsizes].height = 0;
 		for (dmp=papersizes; dmp->name != NULL; dmp++) {
 		    /* Note: Paper size comment uses down cased paper size
 		     * name.  Case insensitive compares are only used for
 		     * PaperSize comments.
 		     */
-		    if (strcasecmp(doc->media[doc->nummedia].name,
+		    if (strcasecmp(doc->size[doc->numsizes].name,
 			       dmp->name) == 0) {
-			g_free(doc->media[doc->nummedia].name);
-			doc->media[doc->nummedia].name = g_strdup (dmp->name);
-			doc->media[doc->nummedia].name = dmp->name;
-			doc->media[doc->nummedia].width = dmp->width;
-			doc->media[doc->nummedia].height = dmp->height;
+			g_free(doc->size[doc->numsizes].name);
+			doc->size[doc->numsizes].name = g_strdup (dmp->name);
+			doc->size[doc->numsizes].name = dmp->name;
+			doc->size[doc->numsizes].width = dmp->width;
+			doc->size[doc->numsizes].height = dmp->height;
 			break;
 		    }
 		}
-		if (doc->media[doc->nummedia].width != 0 &&
-		    doc->media[doc->nummedia].height != 0) doc->nummedia++;
+		if (doc->size[doc->numsizes].width != 0 &&
+		    doc->size[doc->numsizes].height != 0) doc->numsizes++;
 		else
-		    g_free(doc->media[doc->nummedia].name);
+		    g_free(doc->size[doc->numsizes].name);
 	    }
 	    preread=1;
 	    while (readline(line, sizeof line, file, &position, &line_len) &&
@@ -484,31 +489,31 @@ struct document *psscan (FILE *file, int respect_eof)
 		section_len += line_len;
 		next_char = line + length("%%+");
 		while ((cp = get_next_text(next_char, &next_char)) ) {
-		    doc->media = g_renew (GtkGSPaperSize,
-					  doc->media, doc->nummedia+1);
-		    doc->media[doc->nummedia].name = cp;
-		    doc->media[doc->nummedia].width = 0;
-		    doc->media[doc->nummedia].height = 0;
+		    doc->size = g_renew (GtkGSPaperSize,
+					 doc->size, doc->numsizes+1);
+		    doc->size[doc->numsizes].name = cp;
+		    doc->size[doc->numsizes].width = 0;
+		    doc->size[doc->numsizes].height = 0;
 		    for (dmp=papersizes; dmp->name != NULL; dmp++) {
 			/* Note: Paper size comment uses down cased paper size
 			 * name.  Case insensitive compares are only used for
 			 * PaperSize comments.
 			 */
-			if (strcasecmp(doc->media[doc->nummedia].name,
+			if (strcasecmp(doc->size[doc->numsizes].name,
 				   dmp->name) == 0) {
-			    doc->media[doc->nummedia].width = dmp->width;
-			    doc->media[doc->nummedia].height = dmp->height;
+			    doc->size[doc->numsizes].width = dmp->width;
+			    doc->size[doc->numsizes].height = dmp->height;
 			    break;
 			}
 		    }
-		    if (doc->media[doc->nummedia].width != 0 &&
-			doc->media[doc->nummedia].height != 0) doc->nummedia++;
+		    if (doc->size[doc->numsizes].width != 0 &&
+			doc->size[doc->numsizes].height != 0) doc->numsizes++;
 		    else
-			g_free(doc->media[doc->nummedia].name);
+			g_free(doc->size[doc->numsizes].name);
 		}
 	    }
 	    section_len += line_len;
-	    if (doc->nummedia != 0) doc->default_page_media = doc->media;
+	    if (doc->numsizes != 0) doc->default_page_size = doc->size;
 	}
     }
 
@@ -565,17 +570,20 @@ struct document *psscan (FILE *file, int respect_eof)
 		iscomment(line+2, "PageOrientation:")) {
 		sscanf(line+length("%%PageOrientation:"), "%s", text);
 		if (strcmp(text, "Portrait") == 0) {
-		    doc->default_page_orientation = PORTRAIT;
+		    doc->default_page_orientation = GTK_GS_ORIENTATION_PORTRAIT;
 		} else if (strcmp(text, "Landscape") == 0) {
-		    doc->default_page_orientation = LANDSCAPE;
+		    doc->default_page_orientation = GTK_GS_ORIENTATION_LANDSCAPE;
 		}
-	    } else if (page_media_set == NONE &&
+                else if(strcmp(text, "Seascape") == 0) {
+                        doc->default_page_orientation = GTK_GS_ORIENTATION_SEASCAPE;
+                }
+	    } else if (page_size_set == NONE &&
 		       iscomment(line+2, "PageMedia:")) {
 		cp = get_next_text(line+length("%%PageMedia:"), NULL);
-		for (dmp = doc->media, i=0; i<doc->nummedia; i++, dmp++) {
+		for (dmp = doc->size, i=0; i<doc->numsizes; i++, dmp++) {
 		    if (strcmp(cp, dmp->name) == 0) {
-			doc->default_page_media = dmp;
-			page_media_set = 1;
+			doc->default_page_size = dmp;
+			page_size_set = 1;
 			break;
 		    }
 		}
@@ -689,21 +697,24 @@ struct document *psscan (FILE *file, int respect_eof)
 		iscomment(line+2, "PageOrientation:")) {
 		sscanf(line+length("%%PageOrientation:"), "%s", text);
 		if (strcmp(text, "Portrait") == 0) {
-		    doc->default_page_orientation = PORTRAIT;
+		    doc->default_page_orientation = GTK_GS_ORIENTATION_PORTRAIT;
 		} else if (strcmp(text, "Landscape") == 0) {
-		    doc->default_page_orientation = LANDSCAPE;
+		    doc->default_page_orientation = GTK_GS_ORIENTATION_LANDSCAPE;
 		}
-	    } else if (page_media_set == NONE &&
+                else if(strcmp(text, "Seascape") == 0) {
+                        doc->default_page_orientation = GTK_GS_ORIENTATION_SEASCAPE;
+                }
+	    } else if (page_size_set == NONE &&
 		       iscomment(line+2, "PaperSize:")) {
 		cp = get_next_text(line+length("%%PaperSize:"), NULL);
-		for (dmp = doc->media, i=0; i<doc->nummedia; i++, dmp++) {
+		for (dmp = doc->size, i=0; i<doc->numsizes; i++, dmp++) {
 		    /* Note: Paper size comment uses down cased paper size
 		     * name.  Case insensitive compares are only used for
 		     * PaperSize comments.
 		     */
 		    if (strcasecmp(cp, dmp->name) == 0) {
-			doc->default_page_media = dmp;
-			page_media_set = 1;
+			doc->default_page_size = dmp;
+			page_size_set = 1;
 			break;
 		    }
 		}
@@ -828,30 +839,33 @@ continuepage:
 		iscomment(line+2, "PageOrientation:")) {
 		sscanf(line+length("%%PageOrientation:"), "%s", text);
 		if (strcmp(text, "Portrait") == 0) {
-		    doc->pages[doc->numpages].orientation = PORTRAIT;
+		    doc->pages[doc->numpages].orientation = GTK_GS_ORIENTATION_PORTRAIT;
 		} else if (strcmp(text, "Landscape") == 0) {
-		    doc->pages[doc->numpages].orientation = LANDSCAPE;
+		    doc->pages[doc->numpages].orientation = GTK_GS_ORIENTATION_LANDSCAPE;
 		}
-	    } else if (doc->pages[doc->numpages].media == NULL &&
+                else if(strcmp(text, "Seascape") == 0) {
+                        doc->pages[doc->numpages].orientation = GTK_GS_ORIENTATION_SEASCAPE;
+                }
+	    } else if (doc->pages[doc->numpages].size == NULL &&
 		       iscomment(line+2, "PageMedia:")) {
 		cp = get_next_text(line+length("%%PageMedia:"), NULL);
-		for (dmp = doc->media, i=0; i<doc->nummedia; i++, dmp++) {
+		for (dmp = doc->size, i=0; i<doc->numsizes; i++, dmp++) {
 		    if (strcmp(cp, dmp->name) == 0) {
-			doc->pages[doc->numpages].media = dmp;
+			doc->pages[doc->numpages].size = dmp;
 			break;
 		    }
 		}
 		g_free(cp);
-	    } else if (doc->pages[doc->numpages].media == NULL &&
+	    } else if (doc->pages[doc->numpages].size == NULL &&
 		       iscomment(line+2, "PaperSize:")) {
 		cp = get_next_text(line+length("%%PaperSize:"), NULL);
-		for (dmp = doc->media, i=0; i<doc->nummedia; i++, dmp++) {
+		for (dmp = doc->size, i=0; i<doc->numsizes; i++, dmp++) {
 		    /* Note: Paper size comment uses down cased paper size
 		     * name.  Case insensitive compares are only used for
 		     * PaperSize comments.
 		     */
 		    if (strcasecmp(cp, dmp->name) == 0) {
-			doc->pages[doc->numpages].media = dmp;
+			doc->pages[doc->numpages].size = dmp;
 			break;
 		    }
 		}
@@ -974,10 +988,13 @@ continuepage:
 		   iscomment(line+2, "Orientation:")) {
 	    sscanf(line+length("%%Orientation:"), "%s", text);
 	    if (strcmp(text, "Portrait") == 0) {
-		doc->orientation = PORTRAIT;
+		doc->orientation = GTK_GS_ORIENTATION_PORTRAIT;
 	    } else if (strcmp(text, "Landscape") == 0) {
-		doc->orientation = LANDSCAPE;
+		doc->orientation = GTK_GS_ORIENTATION_LANDSCAPE;
 	    }
+            else if(strcmp(text, "Seascape") == 0) {
+                    doc->orientation = GTK_GS_ORIENTATION_SEASCAPE;
+            }
 	} else if (page_order_set == ATEND && iscomment(line+2, "PageOrder:")) {
 	    sscanf(line+length("%%PageOrder:"), "%s", text);
 	    if (strcmp(text, "Ascend") == 0) {
@@ -1056,13 +1073,13 @@ psfree(doc)
 	for (i=0; i<doc->numpages; i++) {
 	    if (doc->pages[i].label) g_free(doc->pages[i].label);
 	}
-	for (i=0; i<doc->nummedia; i++) {
-	    if (doc->media[i].name) g_free(doc->media[i].name);
+	for (i=0; i<doc->numsizes; i++) {
+	    if (doc->size[i].name) g_free(doc->size[i].name);
 	}
 	if (doc->title) g_free(doc->title);
 	if (doc->date) g_free(doc->date);
 	if (doc->pages) g_free(doc->pages);
-	if (doc->media) g_free(doc->media);
+	if (doc->size) g_free(doc->size);
 	g_free(doc);
     }
 }
@@ -1318,12 +1335,19 @@ readline(line, size, fp, position, line_len)
     char *cp;
     unsigned int num;
     unsigned int nbytes;
-    int i;
+    int i, j;
     char buf[BUFSIZ];
 
     if (position) *position = ftell(fp);
     cp = fgets(line, size, fp);
     if (cp == NULL) line[0] = '\0';
+    for(i = 0; line[i] != '\0' && (line[i] == 0x0c || line[i] == ' ' || line[i] == '\t'); i++) 
+            ;
+    if(i > 0 && line[i] == '%' && line[i + 1] == '%') {
+            for(j = i; line[j] != '\0'; j++)
+                    line[j - i] = line[j];
+            line[j - i] = '\0';
+    }
     *line_len = strlen(line);
     if (!(DSCcomment(line) && iscomment(line+2, "Begin"))) {
 	/* Do nothing */
@@ -1432,56 +1456,55 @@ readline(line, size, fp, position, line_len)
 
 void
 pscopy(from, to, begin, end)
-    FILE *from;
-    FILE *to;
-    long begin;			/* set negative to avoid initial seek */
-    long end;
+     FILE *from;
+     GtkGSDocSink *to;
+     long begin;			/* set negative to avoid initial seek */
+     long end;
 {
-    char line[PSLINELENGTH];	/* 255 characters + 1 newline + 1 NULL */
-    char text[PSLINELENGTH];	/* Temporary storage for text */
-    unsigned int num;
-    int i;
-    char buf[BUFSIZ];
+        char line[PSLINELENGTH];	/* 255 characters + 1 newline + 1 NULL */
+        char text[PSLINELENGTH];	/* Temporary storage for text */
+        unsigned int num;
+        int i;
+        char buf[BUFSIZ];
 
-    if (begin >= 0) fseek(from, begin, SEEK_SET);
-    while (ftell(from) < end) {
-
-	fgets(line, sizeof line, from);
-	fputs(line, to);
-
-	if (!(DSCcomment(line) && iscomment(line+2, "Begin"))) {
-	    /* Do nothing */
-	} else if (iscomment(line+7, "Data:")) {
-	    text[0] = '\0';
-	    if (sscanf(line+length("%%BeginData:"),
-		       "%d %*s %s", &num, text) >= 1) {
-		if (strcmp(text, "Lines") == 0) {
-		    for (i=0; i < num; i++) {
-			fgets(line, sizeof line, from);
-			fputs(line, to);
-		    }
-		} else {
-		    while (num > BUFSIZ) {
-			fread(buf, sizeof (char), BUFSIZ, from);
-			fwrite(buf, sizeof (char), BUFSIZ, to);
-			num -= BUFSIZ;
-		    }
-		    fread(buf, sizeof (char), num, from);
-		    fwrite(buf, sizeof (char), num, to);
-		}
-	    }
-	} else if (iscomment(line+7, "Binary:")) {
-	    if(sscanf(line+length("%%BeginBinary:"), "%d", &num) == 1) {
-		while (num > BUFSIZ) {
-		    fread(buf, sizeof (char), BUFSIZ, from);
-		    fwrite(buf, sizeof (char), BUFSIZ, to);
-		    num -= BUFSIZ;
-		}
-		fread(buf, sizeof (char), num, from);
-		fwrite(buf, sizeof (char), num, to);
-	    }
-	}
-    }
+        if (begin >= 0) fseek(from, begin, SEEK_SET);
+        while (ftell(from) < end) {
+                fgets(line, sizeof line, from);
+                gtk_gs_doc_sink_write(to, line, strlen(line));
+                
+                if (!(DSCcomment(line) && iscomment(line+2, "Begin"))) {
+                        /* Do nothing */
+                } else if (iscomment(line+7, "Data:")) {
+                        text[0] = '\0';
+                        if (sscanf(line+length("%%BeginData:"),
+                                   "%d %*s %s", &num, text) >= 1) {
+                                if (strcmp(text, "Lines") == 0) {
+                                        for (i=0; i < num; i++) {
+                                                fgets(line, sizeof(line), from);
+                                                gtk_gs_doc_sink_write(to, line, strlen(line));
+                                        }
+                                } else {
+                                        while (num > BUFSIZ) {
+                                                fread(buf, sizeof (char), BUFSIZ, from);
+                                                gtk_gs_doc_sink_write(to, buf, BUFSIZ);
+                                                num -= BUFSIZ;
+                                        }
+                                        fread(buf, sizeof (char), num, from);
+                                        gtk_gs_doc_sink_write(to, buf, num);
+                                }
+                        }
+                } else if (iscomment(line+7, "Binary:")) {
+                        if(sscanf(line+length("%%BeginBinary:"), "%d", &num) == 1) {
+                                while (num > BUFSIZ) {
+                                        fread(buf, sizeof (char), BUFSIZ, from);
+                                        gtk_gs_doc_sink_write(to, buf, BUFSIZ);
+                                        num -= BUFSIZ;
+                                }
+                                fread(buf, sizeof (char), num, from);
+                                gtk_gs_doc_sink_write(to, buf, num);
+                        }
+                }
+        }
 }
 
 /*
@@ -1492,64 +1515,66 @@ pscopy(from, to, begin, end)
  *		       comments are encountered.
  */
 
-char *pscopyuntil(FILE *from, FILE *to, long begin, long end, const char *comment)
+char *
+pscopyuntil(FILE *from, GtkGSDocSink *to, long begin, long end,
+            const char *comment)
 {
-    char line[PSLINELENGTH];	/* 255 characters + 1 newline + 1 NULL */
-    char text[PSLINELENGTH];	/* Temporary storage for text */
-    unsigned int num;
-    int comment_length;
-    int i;
-    char buf[BUFSIZ];
+        char line[PSLINELENGTH];	/* 255 characters + 1 newline + 1 NULL */
+        char text[PSLINELENGTH];	/* Temporary storage for text */
+        unsigned int num;
+        int comment_length;
+        int i;
+        char buf[BUFSIZ];
 
-    if (comment != NULL)
-      comment_length = strlen(comment);
-    else
-      comment_length = 0;
-    if (begin >= 0) fseek(from, begin, SEEK_SET);
+        if (comment != NULL)
+                comment_length = strlen(comment);
+        else
+                comment_length = 0;
+        if (begin >= 0) fseek(from, begin, SEEK_SET);
+        
+        while (ftell(from) < end && !feof(from)) {
+                fgets(line, sizeof line, from);
 
-    while (ftell(from) < end && !feof(from)) {
-	fgets(line, sizeof line, from);
-
-	/* iscomment cannot be used here,
-	 * because comment_length is not known at compile time. */
-	if (comment != NULL && strncmp(line, comment, comment_length) == 0) {
-	    return g_strdup (line);
-	}
-	fputs(line, to);
-	if (!(DSCcomment(line) && iscomment(line+2, "Begin"))) {
-	    /* Do nothing */
-	} else if (iscomment(line+7, "Data:")) {
-	    text[0] = '\0';
-	    if (sscanf(line+length("%%BeginData:"),
-		       "%d %*s %s", &num, text) >= 1) {
-		if (strcmp(text, "Lines") == 0) {
-		    for (i=0; i < num; i++) {
-			fgets(line, sizeof line, from);
-			fputs(line, to);
-		    }
-		} else {
-		    while (num > BUFSIZ) {
-			fread(buf, sizeof (char), BUFSIZ, from);
-			fwrite(buf, sizeof (char), BUFSIZ, to);
-			num -= BUFSIZ;
-		    }
-		    fread(buf, sizeof (char), num, from);
-		    fwrite(buf, sizeof (char), num, to);
-		}
-	    }
-	} else if (iscomment(line+7, "Binary:")) {
-	    if(sscanf(line+length("%%BeginBinary:"), "%d", &num) == 1) {
-		while (num > BUFSIZ) {
-		    fread(buf, sizeof (char), BUFSIZ, from);
-		    fwrite(buf, sizeof (char), BUFSIZ, to);
-		    num -= BUFSIZ;
-		}
-		fread(buf, sizeof (char), num, from);
-		fwrite(buf, sizeof (char), num, to);
-	    }
-	}
-    }
-    return NULL;
+                /* iscomment cannot be used here,
+                 * because comment_length is not known at compile time. */
+                if (comment != NULL && strncmp(line, comment, comment_length) == 0) {
+                        return g_strdup (line);
+                }
+                gtk_gs_doc_sink_write(to, line, strlen(line));
+                if (!(DSCcomment(line) && iscomment(line+2, "Begin"))) {
+                        /* Do nothing */
+                } else if (iscomment(line+7, "Data:")) {
+                        text[0] = '\0';
+                        if (sscanf(line+length("%%BeginData:"),
+                                   "%d %*s %s", &num, text) >= 1) {
+                                if (strcmp(text, "Lines") == 0) {
+                                        for (i=0; i < num; i++) {
+                                                fgets(line, sizeof line, from);
+                                                gtk_gs_doc_sink_write(to, line, strlen(line));
+                                        }
+                                } else {
+                                        while (num > BUFSIZ) {
+                                                fread(buf, sizeof (char), BUFSIZ, from);
+                                                gtk_gs_doc_sink_write(to, buf, BUFSIZ);
+                                                num -= BUFSIZ;
+                                        }
+                                        fread(buf, sizeof (char), num, from);
+                                        gtk_gs_doc_sink_write(to, buf, num);
+                                }
+                        }
+                } else if (iscomment(line+7, "Binary:")) {
+                        if(sscanf(line+length("%%BeginBinary:"), "%d", &num) == 1) {
+                                while (num > BUFSIZ) {
+                                        fread(buf, sizeof (char), BUFSIZ, from);
+                                        gtk_gs_doc_sink_write(to, buf, BUFSIZ);
+                                        num -= BUFSIZ;
+                                }
+                                fread(buf, sizeof (char), num, from);
+                                gtk_gs_doc_sink_write(to, buf, num);
+                        }
+                }
+        }
+        return NULL;
 }
 
 /*
@@ -1569,99 +1594,97 @@ static int blank(char *line)
 /* Copy the headers, marked pages, and trailer to fp */
 /*##########################################################*/
 
-void pscopydoc(FILE *dest_file,
+void pscopydoc(GtkGSDocSink *dest,
 	       char *src_filename,
 	       struct document *d,
 	       gint *pagelist)
 {
-  FILE *src_file;
-  char text[PSLINELENGTH];
-  char *comment;
-  gboolean pages_written = FALSE;
-  gboolean pages_atend = FALSE;
-  int pages;
-  int page = 1;
-  int i, j;
-  int here;
-  /*
-  FileData fd;
-  */
+        FILE *src_file;
+        char text[PSLINELENGTH];
+        char *comment;
+        gboolean pages_written = FALSE;
+        gboolean pages_atend = FALSE;
+        int pages;
+        int page = 1;
+        int i, j;
+        int here;
   
-  src_file = fopen(src_filename, "r");
-  /*
-  fd = ps_io_init(src_file);
-  */
-  i=0;
-  pages=0;
-  for (i=0;i<d->numpages;i++)    {
-    if (pagelist[i]) pages++;
-  }
+        src_file = fopen(src_filename, "r");
+        i=0;
+        pages=0;
+        for (i=0;i<d->numpages;i++)    {
+                if (pagelist[i]) pages++;
+        }
 
-  here = d->beginheader;
+        here = d->beginheader;
 
-  while ((comment=pscopyuntil(src_file,dest_file,here,d->endheader,"%%Pages:"))) {
+        while ((comment=pscopyuntil(src_file, dest, here, d->endheader,"%%Pages:"))) {
+                here = ftell(src_file);
+                if (pages_written || pages_atend) {
+                        g_free(comment);
+                        continue;
+                }
+                sscanf(comment+length("%%Pages:"), "%s", text);
+                if (strcmp(text, "(atend)") == 0) {
+                        gtk_gs_doc_sink_write(dest, comment, strlen(comment));
+                        pages_atend = TRUE;
+                } else {
+                        switch (sscanf(comment+length("%%Pages:"), "%*d %d", &i)) {
+                        case 1:
+                                gtk_gs_doc_sink_printf(dest, "%%%%Pages: %d %d\n", pages, i);
+                                break;
+                        default:
+                                gtk_gs_doc_sink_printf(dest, "%%%%Pages: %d\n", pages);
+                                break;
+                        }
+                        pages_written = TRUE;
+                }
+                g_free(comment);
+        }
+        pscopyuntil(src_file, dest, d->beginpreview, d->endpreview, NULL);
+        pscopyuntil(src_file, dest, d->begindefaults, d->enddefaults, NULL);
+        pscopyuntil(src_file, dest, d->beginprolog, d->endprolog, NULL);
+        pscopyuntil(src_file, dest, d->beginsetup, d->endsetup, NULL);
 
-       here = ftell(src_file);
-       if (pages_written || pages_atend) {
-          g_free(comment);
-          continue;
-       }
-       sscanf(comment+length("%%Pages:"), "%s", text);
-       if (strcmp(text, "(atend)") == 0) {
-          fputs(comment, dest_file);
-          pages_atend = TRUE;
-       } else {
-          switch (sscanf(comment+length("%%Pages:"), "%*d %d", &i)) {
-             case 1:
-                fprintf(dest_file, "%%%%Pages: %d %d\n", pages, i);
-                break;
-             default:
-                fprintf(dest_file, "%%%%Pages: %d\n", pages);
-                break;
-          }
-          pages_written = TRUE;
-       }
-       g_free(comment);
-  }
-   pscopyuntil(src_file, dest_file, d->beginpreview, d->endpreview,NULL);
-   pscopyuntil(src_file, dest_file, d->begindefaults, d->enddefaults,NULL);
-   pscopyuntil(src_file, dest_file, d->beginprolog, d->endprolog,NULL);
-   pscopyuntil(src_file, dest_file, d->beginsetup, d->endsetup,NULL);
+        for (i = 0; i < d->numpages; i++) {
+                if (d->pageorder == DESCEND)
+                        j = (d->numpages - 1) - i;
+                else
+                        j = i;
+                j = i;
+                if (pagelist[j]) {
+                        comment = pscopyuntil(src_file, dest,
+                                              d->pages[i].begin,
+                                              d->pages[i].end,
+                                              "%%Page:");
+                        gtk_gs_doc_sink_printf(dest, "%%%%Page: %s %d\n",
+                                               d->pages[i].label, page++);
+                        g_free(comment);
+                        pscopyuntil(src_file, dest, -1, d->pages[i].end, NULL);
+                }
+        }
 
-   for (i = 0; i < d->numpages; i++) {
-     if (d->pageorder == DESCEND) j = (d->numpages - 1) - i;
-     else                         j = i;
+        here = d->begintrailer;
+        while ((comment = pscopyuntil(src_file, dest, here, d->endtrailer,
+                                      "%%Pages:"))) {
+                here = ftell(src_file);
+                if (pages_written) {
+                        g_free(comment);
+                        continue;
+                }
+                switch (sscanf(comment+length("%%Pages:"), "%*d %d", &i)) {
+                case 1:
+                        gtk_gs_doc_sink_printf(dest, "%%%%Pages: %d %d\n",
+                                               pages, i);
+                        break;
+                default:
+                        gtk_gs_doc_sink_printf(dest, "%%%%Pages: %d\n",
+                                               pages);
+                        break;
+                }
+                pages_written = TRUE;
+                g_free(comment);
+        }
 
-     j = i;
-     if (pagelist[j]) {
-
-       comment = pscopyuntil(src_file,dest_file,
-			     d->pages[i].begin,d->pages[i].end, "%%Page:");
-       fprintf(dest_file, "%%%%Page: %s %d\n",d->pages[i].label, page++);
-       g_free(comment);
-       pscopyuntil(src_file, dest_file, -1, d->pages[i].end,NULL);
-     }
-   }
-
-   here = d->begintrailer;
-   while ((comment = pscopyuntil(src_file, dest_file, here, d->endtrailer, "%%Pages:"))) {
-      here = ftell(src_file);
-      if (pages_written) {
-         g_free(comment);
-         continue;
-      }
-      switch (sscanf(comment+length("%%Pages:"), "%*d %d", &i)) {
-         case 1:
-            fprintf(dest_file, "%%%%Pages: %d %d\n", pages, i);
-            break;
-         default:
-            fprintf(dest_file, "%%%%Pages: %d\n", pages);
-            break;
-      }
-      pages_written = TRUE;
-      g_free(comment);
-   }
-
-   fclose(dest_file);
-   fclose(src_file);
+        fclose(src_file);
 }
